#include "installer.h"
#include <unistd.h>
#include <signal.h>
#include "partition.h"

#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <linux/fs.h>
#include <errno.h>


/* global variables */
struct tgt_object *list_p3, *label1_p3, *label2_p3, *string1_p3, *b1_p3, *b2_p3, *b3_p3, *line_p3, *columns_p3;
struct tgt_object *p_window, *p_label1, *p_label2, *p_label3, *p_check1, 
				  *p_string1, *p_string2, *p_b1, *p_b2,
				  *p_select1, *p_select2, *p_line;
struct htpt {
		int h;
		int p;
} hppos[10];

int pt[10], mountpos, formatpos;

char *mounts[] = {"nincs","/","/boot","/home","/opt","/tmp","/usr","/usr/local",
"/usr/src","/var","/var/cache","/var/cvs","/var/ftp","/var/log",
"/var/spool","/var/tmp","/var/www","swap",NULL};

char *formats[] = {"nincs","ext3","ext2","reiserfs","jfs","xfs","fat32","fat16","swap",NULL};

void step_part3();
int part1_main();
int pack_main();

/* functions */
void part3_prev() {
	tgt_unlink(window);
	part1_main();
	tgt_refresh(window);
}

void part3_next() {
	tgt_unlink(window);
	pack_main();
	tgt_refresh(window);
}

void pwinquit() {
	tgt_unlink(p_window);
	tgt_destroyobject(p_window);
}

void pwinsave() {
	 int s1,s2,c1,ph,pp;
	 
	 pp = hppos[selectedhdd].p;
	 ph = hppos[selectedhdd].h;
	 
	 tgt_get(p_select1,TGTT_SELECTBOX_SELECTED,(tgtt*) &s1);
	 tgt_get(p_select2,TGTT_SELECTBOX_SELECTED,(tgtt*) &s2);
	 tgt_get(p_check1,TGTT_CHECKBOX_ACTIVE,(tgtt*) &c1);
	 
	 if ( s1 != 0 ) { 
	 
	  sprintf(hddparts[ph].partmount[pp],"%s",mounts[s1]);
	  sprintf(hddparts[ph].partformat[pp],"%s",formats[s2]);
	 
	  if (c1 == 1) { hddparts[ph].pformat[pp] = 1; 
	  } else { hddparts[ph].pformat[pp] = 0;  }
	 
	 }
	 
	 tgt_unlink(p_window);
	 tgt_destroyobject(p_window); 	 
}

void checkform() {
	int s1, s2, i, j, nocheck = 0;
		
	tgt_get(p_select1,TGTT_SELECTBOX_SELECTED,(tgtt*) &s1);
	tgt_get(p_select2,TGTT_SELECTBOX_SELECTED,(tgtt*) &s2);
	
	/* egy mountpointot csak egyszer lehessen kivalasztani */
	for (i=0; i<db; i++) {
	   for (j=0; j<hddparts[i].partnum; j++) {
	   
	 	if ( !(strcmp(hddparts[i].partmount[j],mounts[s1])) ) {
		
			tgt_easyrequest(p_window,"Figyelmeztets!","Mr van ilyen csatolsi pontod!","[ Ok ]");
			tgt_set(p_select1,TGTT_SELECTBOX_SELECTED, (tgtt) 0);
			tgt_set(p_select2,TGTT_SELECTBOX_SELECTED, (tgtt) 0);
			tgt_set(p_check1,TGTT_CHECKBOX_ACTIVE,0);
			tgt_refresh(p_window);
			
			nocheck = 1;
				
		} 	 
	   
	 }
	}	
	
	if ( !nocheck ) {
			if ( !(strcmp(mounts[s1],"swap")) && (strcmp(formats[s2],"swap")) ) {
	 		 tgt_set(p_select2,TGTT_SELECTBOX_SELECTED,8);
	 		 tgt_refresh(p_select2);
			} else if ( !(strcmp(mounts[s1],"/")) && (strcmp(formats[s2],"ext3")) ) {
	 		 tgt_set(p_select2,TGTT_SELECTBOX_SELECTED,1);
	 		 tgt_refresh(p_select2);
			} 
	
			if ( !(strcmp(mounts[s1],"/")) || !(strcmp(mounts[s1],"swap")) ) {
	 		 tgt_set(p_check1,TGTT_CHECKBOX_ACTIVE,1);
	 		 tgt_refresh(p_check1);
			}
	 }
}
/* partition type window */
char spart[100];
void part_window(int pp, int ph) {
	int i;

	sprintf(spart,"%s - %s",hddparts[ph].parttype[pp],hddparts[ph].partsize[pp]);	
	
	p_window = tgt_createobject(
	TGT_CLASS_WINDOW, (tgtt[]) {
	TGTT_WINDOW_TITLE, (tgtt) "Partci tulajdonsgai",
	TGTT_WINDOW_TITLECOLOR,3,
	TGTT_WINDOW_BORDERCOLOR,3,
	TGTT_CALLBACK, (tgtt) hotkeys,
	TGTT_XS, 55, TGTT_YS, 12,	
	TGTT_END,0 
	});
	
	tgt_set(p_window, TGTT_X, (tgt_getterminal()->x_size - 55) / 2);
	tgt_set(p_window, TGTT_Y, (tgt_getterminal()->y_size - 12) / 2);	
	
	tgt_link(p_window,desktop);
	
	p_label3 = tgt_createandlink(p_window,TGT_CLASS_LABEL,
	(tgtt[]) {TGTT_X,2,TGTT_Y,2,
	TGTT_LABEL_TEXT, (tgtt) spart,	
	TGTT_END,0});
	
	p_label1 = tgt_createandlink(p_window,TGT_CLASS_LABEL,
	(tgtt[]) {TGTT_X,2,TGTT_Y,4,
	TGTT_LABEL_TEXT, (tgtt) "Csatolsi pont:",	
	TGTT_END,0});
	
	p_select1 = tgt_createandlink(p_window,TGT_CLASS_SELECTBOX,(tagitem[]) {
	TGTT_X,18,TGTT_Y,4,TGTT_XS,22,
	TGTT_SELECTBOX_MARK1, (tgtt) "<",
	TGTT_SELECTBOX_MARK2, (tgtt) ">",
	TGTT_SELECTBOX_ITEMS, (tgtt) mounts,
	TGTT_CALLBACK, (tgtt) checkform,
	TGTT_END,0});
	
	/* i < mounts szama */
	for (i = 0; i<18; i++) {
	 if ( !(strcmp(hddparts[ph].partmount[pp],mounts[i])) ) {
	  tgt_set(p_select1,TGTT_SELECTBOX_SELECTED,i);
	  tgt_refresh(p_select1);
	 }
	}
	
	p_label2 = tgt_createandlink(p_window,TGT_CLASS_LABEL,
	(tgtt[]) {TGTT_X,2,TGTT_Y,5,
	TGTT_LABEL_TEXT, (tgtt) "Fjlrendszer:",	
	TGTT_END,0});
	
	p_select2 = tgt_createandlink(p_window,TGT_CLASS_SELECTBOX,(tagitem[]) {
	TGTT_X,18,TGTT_Y,5,TGTT_XS,22,
	TGTT_SELECTBOX_MARK1, (tgtt) "<",
	TGTT_SELECTBOX_MARK2, (tgtt) ">",
	TGTT_SELECTBOX_ITEMS, (tgtt) formats, 
	TGTT_CALLBACK, (tgtt) checkform,
	TGTT_END,0});
	
	/* i < mint formats szama */
	for (i = 0; i<9; i++) {
	 if ( !(strcmp(hddparts[ph].partformat[pp],formats[i])) ) {
	  tgt_set(p_select2,TGTT_SELECTBOX_SELECTED,i);
	  tgt_refresh(p_select2);
	 }
	}
	
	p_check1 = tgt_createandlink(p_window,TGT_CLASS_CHECKBOX,(tgtt[]) {
	TGTT_X,2,TGTT_Y,6,
	TGTT_CHECKBOX_CAPTION, (tgtt) "Formzand",
	TGTT_CHECKBOX_ACTIVE,1, 
	TGTT_END,0});
	
	if (hddparts[ph].pformat[pp] == 1 || !(strcmp(hddparts[ph].partmount[pp],"/")) || !(strlen(hddparts[ph].partmount[pp])) ) {
	 tgt_set(p_check1,TGTT_CHECKBOX_ACTIVE,1);
	 tgt_refresh(p_check1);
	} else {
	 tgt_set(p_check1,TGTT_CHECKBOX_ACTIVE,0);
	 tgt_refresh(p_check1);
	}
	
	p_line = tgt_createandlink(p_window,TGT_CLASS_LABEL,
	(tgtt[]) {TGTT_X,1,TGTT_Y,9,
	TGTT_LABEL_TEXT, (tgtt) "-----------------------------------------------------",	
	TGTT_END,0});

	p_b2 = tgt_createandlink(p_window,TGT_CLASS_BUTTON,(tgtt[]) {
	TGTT_X,35,TGTT_Y,10,
	TGTT_BUTTON_CAPTION, (tgtt) "  Ok  ",
	TGTT_CALLBACK, (tgtt) pwinsave,
	TGTT_BG,1,
	TGTT_END });
	
	p_b1 = tgt_createandlink(p_window,TGT_CLASS_BUTTON,(tgtt[]) {
	TGTT_X,12,TGTT_Y,10,
	TGTT_BUTTON_CAPTION, (tgtt) " Mgse ",
	TGTT_CALLBACK, (tgtt) pwinquit,
	TGTT_BG,1,
	TGTT_END });
	
	tgt_setfocus(p_window);
	tgt_refresh(desktop);
}

/* main functions */
int listfill_part3(void **d, int c, int n, char *tmp, int k, int a) {
	int i = 0,j = 0, psz = 0;
	char partlist[20][200];
			
 	if(c == TGT_LISTREQ_GETMAX) {
	 return(partdb);	
	} else { 
		 
	 for (i = 0; i<db; i++) {
	  sprintf(partlist[psz],"%s - %s",hddparts[i].hddname,hddparts[i].hddsize);
	  pt[psz] = 1; hppos[psz].h = i; psz++;
	   for(j = 0; j<hddparts[i].partnum; j++) {
	     sprintf(partlist[psz],"\tpart%d - %s - %s   [%s] [%s] [%d]",j+1,hddparts[i].parttype[j],hddparts[i].partsize[j],hddparts[i].partmount[j],hddparts[i].partformat[j],hddparts[i].pformat[j]);
		 pt[psz] = 2; hppos[psz].h = i; hppos[psz].p = j; psz++;
		}
	   }
	 snprintf(tmp,k,partlist[n],partlist[psz]);  
	 
	 if ( a == 1 ) { selectedhdd = n; }
	 	 	  
   }  	 
   return(1); 
} 

void listdo_part3() {
	int p, pp, ph,i, pid, status; 
	char sdev[120];
	int fd;
	FILE *f;
			
	if ( pt[selectedhdd] == 1 ) { 
	
	  p = hppos[selectedhdd].h;
	/*   if ((pid = fork()) <0 ) { popupmsg("Hiba","Nem tudtam forkolni!\n\n"); off(); }
	  if ( !pid ) { 
	   sprintf(sdev,"openvt -s -w cfdisk %s",hddparts[p].hdddev);
	   system(sdev);
	   exit(0);
	   } while (wait(&status) != pid); */
	   
	  f = fopen("/tmp/part3.sh","w");
	  fprintf(f,"#!/bin/bash\n\ncfdisk %s\nsfdisk -R %s\nexec ./uhu-console-installer -p\nexit",hddparts[p].hdddev,hddparts[p].hdddev);
	  fclose(f);
	  system("chmod a+x /tmp/part3.sh");
	  execl("/tmp/part3.sh",NULL); 
	  
	  partdb = 0;
	  if ( (fd = open(hddparts[p].hdddev,O_RDONLY | O_NONBLOCK)) ) { popupmsg("Hiba",hddparts[p].hdddev); }
	  if (ioctl(fd,BLKRRPART)) { popupmsg("Hiba","Nem tudtam ujraolvasni a particiokat\n\n"); }
	  getdevs(); 	  
	  
	  for (i = 0; i < db; i++) {
	   getparts(i);
      } partdb = partdb+db; 
	  getsize();
	  tgt_refresh(list_p3);
	 	 
	 } else {
	  pp = hppos[selectedhdd].p;
	  ph = hppos[selectedhdd].h;
	  part_window(pp,ph);
	 }
	
}

void saveparts() {
	int i,j, pn = 1, rootcheck = 0;
	char pnt[20], tmp[200];
	char *target;
	
	target = getenv("TARGET");
	
	
	conf = tgt_load_conf(confile);
	
	
	 for (i=0; i<db; i++) {
	   for (j=0; j<hddparts[i].partnum; j++) {
	   
	    if ( (strlen(hddparts[i].partmount[j])) !=0) {
		
		 if ( !(strcmp(hddparts[i].partmount[j],"/")) ) { sprintf(pnt,"root"); rootcheck = 1;
		 } else if ( !(strcmp(hddparts[i].partmount[j],"swap")) ) { sprintf(pnt,"swap"); }
		 else { sprintf(pnt,"part%d",pn); pn++; }
		 
		 sprintf(tmp,"%s,%s,%s,%d",hddparts[i].part[j],hddparts[i].partmount[j],hddparts[i].partformat[j],hddparts[i].pformat[j]);
		 tgt_set_conf(conf,"part3",pnt,tmp);
		 tgt_save_conf(conf,confile);
		 
	    }		
	   
	   }	   
	  } tgt_unload_conf(conf); 
	  
	  if ( !rootcheck ) { popupmsg("Figyelmeztets","Nem adtl meg \"/\" partcit!\n\n"); }
	  else { part3_next(); }
   
}

void step_part3 () {
	
	window = tgt_createobject(
	TGT_CLASS_WINDOW, (tgtt[]) {
	TGTT_WINDOW_TITLE, (tgtt) "UHU-Linux - Partcionls - ltalam vlasztott partcira",
	TGTT_WINDOW_TITLECOLOR,3,
	TGTT_WINDOW_BORDERCOLOR,3,
	TGTT_CALLBACK, (tgtt) hotkeys,
	TGTT_XS, 78, TGTT_YS, 23,	
	TGTT_END,0 
	});
	
	tgt_set(window, TGTT_X, (tgt_getterminal()->x_size - 78) / 2);
	tgt_set(window, TGTT_Y, (tgt_getterminal()->y_size - 23) / 2);	
	
	tgt_link(window,desktop);
	
	columns_p3 = tgt_createandlink(window,TGT_CLASS_LABEL,
	(tgtt[]) {TGTT_X,3,TGTT_Y,2,
	TGTT_LABEL_TEXT, (tgtt) "[hdd/partci] [mret] [csatolsi pont] [fjlrendszer] [formzand]",	
	TGTT_FG,6,
	TGTT_END,0});
		
	list_p3 = tgt_createandlink(window,TGT_CLASS_LIST,(tgtt[]) {
	TGTT_X,2,TGTT_Y,3,
	TGTT_XS,74,TGTT_YS,10,
	TGTT_LIST_ACTIVEBG,1,
	TGTT_LIST_FRAMECOLOR,3,
	TGTT_ID,1000,
	TGTT_LIST_DATACALLBACK, (tgtt) listfill_part3, 
	TGTT_CALLBACK,(tgtt) listdo_part3,
	TGTT_END,0
	});
	
	label1_p3 = tgt_createandlink(window,TGT_CLASS_LABEL,
	(tgtt[]) {TGTT_X,2,TGTT_Y,14,
	TGTT_LABEL_TEXT, (tgtt) "Ha a merevlemezen nyomsz entert, akkor partcionlni tudod,",	
	TGTT_FG,6,
	TGTT_END,0});
	
	tgt_createandlink(window,TGT_CLASS_LABEL,
	(tgtt[]) {TGTT_X,2,TGTT_Y,15,
	TGTT_LABEL_TEXT, (tgtt) "a cfdisk programmal.",	
	TGTT_FG,6,
	TGTT_END,0});
	
	label2_p3 = tgt_createandlink(window,TGT_CLASS_LABEL,
	(tgtt[]) {TGTT_X,2,TGTT_Y,17,
	TGTT_LABEL_TEXT, (tgtt) "A partcikon nyomott enterrel a partcit tudod belltani.",	
	TGTT_FG,6,
	TGTT_END,0});

	line_p3 = tgt_createandlink(window,TGT_CLASS_LABEL,
	(tgtt[]) {TGTT_X,1,TGTT_Y,20,
	TGTT_LABEL_TEXT, (tgtt) "----------------------------------------------------------------------------",	
	TGTT_END,0});

		
	b1_p3 = tgt_createandlink(window,TGT_CLASS_BUTTON,(tgtt[]) {
	TGTT_X,16,TGTT_Y,21,
	TGTT_BUTTON_CAPTION, (tgtt) " Vissza ",
	TGTT_CALLBACK, (tgtt) part3_prev,
	TGTT_BG,1,
	TGTT_END });
	
	b2_p3 = tgt_createandlink(window,TGT_CLASS_BUTTON,(tgtt[]) {
	TGTT_X,58,TGTT_Y,21,
	TGTT_BUTTON_CAPTION, (tgtt) "  Ok  ",
	TGTT_CALLBACK, (tgtt) saveparts,
	TGTT_BG,1,
	TGTT_END });
	
}

/* main */
int part3_main () {
	int i;
	helpnum = 5;
	
	partdb = 0;
			
	getdevs();
	getmodelsname(); 
	getswap();
		  	  
	for (i = 0; i < db; i++) { getparts(i); } 
	partdb = partdb+db; 
	getsize();
	
    step_part3(); 
	
	return(0);
	 
}
